unit cxdfindop;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Spin, ExtCtrls, ComCtrls, Buttons;

type
  TfFindOptions = class(TForm)
    Label1: TLabel;
    cbFindSubstring: TComboBox;
    Label2: TLabel;
    cbReplacement: TComboBox;
    Label3: TLabel;
    seStartPos: TSpinEdit;
    Label4: TLabel;
    seLoopCount: TSpinEdit;
    ckIgnoreCase: TCheckBox;
    ckKeepWorkFiles: TCheckBox;
    btCancel: TButton;
    btOK: TButton;
    btSave: TButton;
    grLocation: TRadioGroup;
    cbFilename: TComboBox;
    btOpen: TBitBtn;
    OpenDialog1: TOpenDialog;
    OpenDialog2: TOpenDialog;
    procedure btOKClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cbFindSubstringChange(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure btSaveClick(Sender: TObject);
    procedure btOpenClick(Sender: TObject);
    procedure cbFilenameEnter(Sender: TObject);

  private
    procedure Init;
    procedure LoadSettings(const Filename: TfileName; Referesh: boolean = TRUE);
    procedure SaveSettings(const Filename: TfileName);
    procedure ShowGlobalSettings; // display globalsettings
    procedure StoreDisplayedSettings; // store displayed settings to globalsettings
  public
    FileContents: string;

  end;

  tmyControl = class(tControl)
    property Color;
    property ParentColor;
    property Caption;
  end;

  tmywinControl = class(twinControl);

var
  fFindOptions: TfFindOptions;

procedure SetMultiLineCaption(Control: TControl; const CRLF_TAG: char = '|');
implementation
uses cxdconsts; //, inifiles;
{$R *.DFM}

function StrOfEnabledContestant(Contestants: TContestants): string; forward;
function StrOfCheckedMethodes(ContestantMethodes: TContestantMethodes): string; forward;
function StrToEnabledContestant(Contestants: string): TContestants; forward;
function StrToStrCheckedMethodes(ContestantMethodes: string): TContestantMethodes; forward;

const
  COpenSettings = 'OpenSettings';
  CSaveSettings = 'SaveSettings';

var
  dlgOpenSettings: TOpenDialog;
  dlgSaveSettings: TSaveDialog;

procedure saveComboLines(combo: TComboBox; const PermitBlank: boolean = FALSE); forward;

procedure SetMultiLineCaption(Control: TControl; const CRLF_TAG: char = '|');
var
  bs: cardinal;
  cap: string;
begin
  if Control is TWinControl then with tmywinControl(Control) do begin
      cap := Caption;
      if pos(CRLF_TAG, cap) > 0 then begin
        while pos(CRLF_TAG, cap) > 1 do cap[pos(CRLF_TAG, cap)] := ^J;
        Caption := cap;
        bs := GetWindowLong(handle, GWL_STYLE);
        SetWindowLong(handle, GWL_STYLE, bs or BS_MULTILINE);
      end;
    end;
end;

procedure saveComboLines(combo: TComboBox; const PermitBlank: boolean = FALSE);
begin
  with combo.items, combo do
    if (text <> '') and (IndexOf(text) < 0) then begin
      if count > DropDownCount then delete(count - 1);
      Insert(0, text)
    end;
end;

procedure TfFindOptions.btOKClick(Sender: TObject);
begin
  saveComboLines(cbFindSubstring);
  saveComboLines(cbReplacement, TRUE);
  saveComboLines(cbFilename);
  if grLocation.ItemIndex = 0 then begin
    if not fileexists(cbFilename.Text) then begin
      modalresult := mrnone;
      raise exception.Create('file ' + cbFileName.Text + ' is not exist');
    end;
  end;
end;

type
  tDivJoint = (djNone, djDivide, djJoin);

function cxdopdivjointext(const S: string; const dj: tdivjoint): string;
const
  divjoinblock = 64;
  crlf = #13#10;
  function validate: string;
  var
    i: integer;
  begin
    Result := '';
    for i := 1 to length(S) do
      Result := result;
  end;
var
  i: integer;
  SLen: integer;
begin
  Result := '';
  SLen := length(S);
  if SLen < divjoinblock then Result := S
  else begin
    case dj of
      djDivide:
        for i := 0 to length(S) div divjoinblock - 1 do
          Result := Result + copy(S, 1 + (i * divjoinblock), divjoinblock) + crlf;
      djJoin:
        for i := 0 to length(S) div divjoinblock - 1 do
          Result := Result + copy(S, 1 + (i * length(crlf)) + (i * divjoinblock), divjoinblock)
      else ;
    end;
  end;
end;

procedure tfFindOptions.ShowGlobalSettings;
begin
  cbFindSubstring.Text := GlobalSettings.SubStrText;
  cbReplacement.Text := GlobalSettings.ReplacementText;
  cbFilename.Text := GlobalSettings.FileNameText;

  cbFindSubstring.Items.CommaText := GlobalSettings.SubStrList;
  cbReplacement.Items.CommaText := GlobalSettings.ReplacementList;
  cbFilename.Items.CommaText := GlobalSettings.FileNameList;

  dlgSaveSettings.FileName := GlobalSettings.SaveConfig;

  seStartPos.Value := GlobalSettings.StartPos;
  seLoopCount.Value := GlobalSettings.LoopCount;
  grLocation.ItemIndex := GlobalSettings.Location;

  ckIgnoreCase.Checked := GlobalSettings.IgnoreCase;
  ckKeepWorkFiles.Checked := GlobalSettings.KeepWorkFiles;
end;

procedure TfFindOptions.LoadSettings(const Filename: TfileName; Referesh: boolean = TRUE);
const
  hexprefix = '$';
var
  S, sn: string;
  CList: TStringList;
  buf: array[byte] of byte;
  i, L, n: integer;


begin
  if not fileexists(CONFIG_FILE) then begin
  end
  else begin
    CList := TstringList.Create;
    try
      CList.LoadFromFile(CONFIG_FILE);
      if (CList.Count > 1) and (CList.Strings[0] = CONFIG_HEADER) then begin
        GlobalSettings.SubStrText := CList[_ciSubStrText];
        GlobalSettings.ReplacementText := CList[_ciReplacementText];
        GlobalSettings.FileNameText := CList[_ciFileNameText];

        GlobalSettings.SubStrList := CList[_ciSubStrList];
        GlobalSettings.ReplacementList := CList[_ciReplacementList];
        GlobalSettings.FileNameList := CList[_ciFileNameList];

        GlobalSettings.SaveConfig := CList[_ciSaveConfig];

        GlobalSettings.IgnoreCase := boolean(StrToIntDef(CList[_ciIgnoreCase], 0));
        GlobalSettings.KeepWorkFiles := boolean(StrToIntDef(CList[_ciKeepWorkFiles], 0));

        GlobalSettings.StartPos := StrToIntDef(CList[_ciStartPos], 1);
        GlobalSettings.LoopCount := StrToIntDef(CList[_ciLoopCount], 1);
        GlobalSettings.Location := StrToIntDef(CList[_ciLocation], 0);

        S := CList[_ciEnabledContestants];
        L := minof(length(S) div 2, sizeof(GlobalSettings.EnabledContestants));
        fillchar(buf, sizeof(buf), 0);
        for i := 0 to L - 1 do begin
          sn := hexprefix + copy(S, (i * 2) + 1, 2);
          n := StrToIntDef(sn, 0);
          buf[i] := byte(n);
        end;
        move(buf, GlobalSettings.EnabledContestants, L);

        S := CList[_ciCheckedMethodes];
        L := minof(length(S) div 2, sizeof(GlobalSettings.CheckedMethodes));
        fillchar(buf, sizeof(buf), 0);
        for i := 0 to L - 1 do begin
          sn := hexprefix + copy(S, (i * 2) + 1, 2);
          n := StrToIntDef(sn, 0);
          buf[i] := byte(n);
        end;
        move(buf, GlobalSettings.CheckedMethodes, L);
      end;

      if Referesh then ShowGlobalSettings;
    finally
      CList.Free
    end;
  end
end;

procedure tfFindOptions.StoreDisplayedSettings;
begin
  GlobalSettings.SubStrText := cbFindSubstring.Text;
  GlobalSettings.ReplacementText := cbReplacement.Text;
  GlobalSettings.FileNameText := cbFilename.Text;

  GlobalSettings.SubStrList := cbFindSubstring.Items.CommaText;
  GlobalSettings.ReplacementList := cbReplacement.Items.CommaText;
  GlobalSettings.FileNameList := cbFilename.Items.CommaText;

  GlobalSettings.SaveConfig := dlgSaveSettings.FileName;

  GlobalSettings.StartPos := seStartPos.Value;
  GlobalSettings.LoopCount := seLoopCount.Value;
  GlobalSettings.Location := grLocation.ItemIndex;

  GlobalSettings.KeepWorkFiles := ckKeepWorkFiles.Checked;
  GlobalSettings.IgnoreCase := ckIgnoreCase.Checked;
end;

procedure TfFindOptions.SaveSettings(const Filename: TfileName);
const
  fillin = '~';
var
  S: string;
  CList: TStringList;
  buf: array[byte] of byte;
  i: integer;

begin
  CList := TstringList.Create;
  try
    StoreDisplayedSettings;
    for i := 1 to ConfigIndexCount do CList.Add(fillin);
    CList[ord(ciHeader)] := CONFIG_HEADER;

    CList[_ciSubStrText] := GlobalSettings.SubStrText;
    CList[_ciReplacementText] := GlobalSettings.ReplacementText;
    CList[_ciFileNameText] := GlobalSettings.FileNameText;

    CList[_ciSubStrList] := GlobalSettings.SubStrList;
    CList[_ciReplacementList] := GlobalSettings.ReplacementList;
    CList[_ciFileNameList] := GlobalSettings.FileNameList;

    CList[_ciSaveConfig] := GlobalSettings.SaveConfig;

    CList[_ciIgnoreCase] := inttostr(ord(GlobalSettings.IgnoreCase));
    CList[_ciKeepWorkFiles] := inttostr(ord(GlobalSettings.KeepWorkFiles));

    CList[_ciStartPos] := inttostr(GlobalSettings.StartPos);
    CList[_ciLoopCount] := inttostr(GlobalSettings.LoopCount);
    CList[_ciLocation] := inttostr(GlobalSettings.Location);

    i := sizeof(GlobalSettings.EnabledContestants);
    fillchar(buf, sizeof(buf), 0);
    move(GlobalSettings.EnabledContestants, buf, i);
    S := ''; for i := 0 to i - 1 do S := S + inttohex(buf[i], 2);
    CList[_ciEnabledContestants] := S;

    i := sizeof(GlobalSettings.CheckedMethodes);
    fillchar(buf, sizeof(buf), 0);
    move(GlobalSettings.CheckedMethodes, buf, i);
    S := ''; for i := 0 to i - 1 do S := S + inttohex(buf[i], 2);
    CList[_ciCheckedMethodes] := S;

    CList.SaveToFile(FileName);

  finally
    CList.free;
  end;
end;

procedure TfFindOptions.Init;

  procedure InitCombos;
  const
    MAX_COMBO_ITEMS = 17;
  var
    i: integer;
  begin
    for i := 0 to ControlCount - 1 do
      if Controls[i] is TComboBox then
        with tComboBox(Controls[i]) do
          DropDownCount := MAX_COMBO_ITEMS;
  end;
  procedure InitSpinEdits; begin
    with seStartPos do begin
      MaxValue := Maxint;
      MinValue := -MaxInt;
    end;
  end;

  procedure CreateOpenSaveDialog;
  begin
    CONFIG_FILE := Application.ExeName;
    //CONFIG_FILE := extractfilename(CONFIG_FILE);
    CONFIG_FILE := changefileext(CONFIG_FILE, '.ini');

    dlgOpenSettings := TOpenDialog.create(Self);
    with dlgOpenSettings do begin
      filename := CONFIG_FILE;
    end;
    dlgSaveSettings := TSaveDialog.create(Self);
    with dlgSaveSettings do begin
      filename := CONFIG_FILE;
    end;
  end;

begin
  InitCombos;
  InitSpinEdits;
  //ckInplace.Free;
  SetMultiLineCaption(ckIgnoreCase);
  SetMultiLineCaption(ckKeepWorkFiles);
  cbFindSubstring.OnChange(cbFindSubstring);
  CreateOpenSaveDialog;
  LoadSettings(CONFIG_FILE);
end;

procedure TfFindOptions.FormCreate(Sender: TObject);
begin
  KeyPreview := TRUE;
  OnKeyPress := FormKeyPress;
  //not work in D7? Position := poScreenCenter;
  Init;
end;

procedure TfFindOptions.cbFindSubstringChange(Sender: TObject);
var
  blank: boolean;
begin
  exit;
  blank := tcombobox(sender).text = '';
  cbReplacement.Enabled := not blank;
  btOK.Enabled := not blank;
end;

procedure TfFindOptions.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if key = ^[then begin
    key := #0;
    ModalResult := mrCancel;
    Close;
  end;
end;

procedure TfFindOptions.btSaveClick(Sender: TObject);
begin
  //with dlgSaveSettings do
  //  if Execute then begin
  saveComboLines(cbFindSubstring);
  saveComboLines(cbReplacement, TRUE);
  saveComboLines(cbFilename);
  //SaveSettings(FileName);
  //~StoreDisplayedSettings;
  SaveSettings(CONFIG_FILE);
  //  end;
end;

procedure TfFindOptions.btOpenClick(Sender: TObject);
begin
  with OpenDialog1 do if execute then begin
      cbFilename.Text := FileName;
      grLocation.ItemIndex := 0;
    end;
end;

procedure TfFindOptions.cbFilenameEnter(Sender: TObject);
begin
  grLocation.ItemIndex := 0;
end;

function StrOfEnabledContestant(Contestants: TContestants): string;
var
  c: TContestant;
begin
  setlength(Result, ContestantCount);
  for c := low(c) to high(c) do
    //Result := inttohex(ord(c));

end;

function StrOfCheckedMethodes(ContestantMethodes: TContestantMethodes): string;
begin
  //inttohex
end;

function StrToEnabledContestant(Contestants: string): TContestants;
begin
end;

function StrToStrCheckedMethodes(ContestantMethodes: string): TContestantMethodes;
begin
end;

end.

